@@ -481,8 +481,7 @@ transformKernelSignature(Function &F, const std::vector<SPIRVArgDesc> &Descs) {
                        return PointerType::get(Type::getInt8Ty(Ctx), AddrSpace);
                      }
 #endif
-                     if (!VCINTR::Type::isOpaquePointerTy(ArgTy) &&
-                         Arg.hasByValAttr())
+                     if (Arg.hasByValAttr())
                        return OrigTy;
 
                      return ArgTy;
@@ -523,8 +522,6 @@ transformKernelSignature(Function &F, const std::vector<SPIRVArgDesc> &Descs) {
     NewF->addParamAttr(i, Attr);
   }
 
-  legalizeParamAttributes(NewF);
-
   return NewF;
 }
 
@@ -606,6 +603,8 @@ static void rewriteKernelArguments(Function &F) {
     }
   }
 
+  legalizeParamAttributes(NewF);
+
   F.mutateType(NewF->getType());
   F.replaceAllUsesWith(NewF);
   F.eraseFromParent();
